@@ -44,7 +44,7 @@ static char *error_string[] =
   };
 
 int
-otf__error (int err, char *fmt, void *arg)
+otf__error (int err, const char *fmt, const void *arg)
 {
   sprintf (error_message, "OTF-Error (%s): ", error_string[err]);
   sprintf (error_message + strlen (error_message), fmt, arg);
@@ -53,7 +53,7 @@ otf__error (int err, char *fmt, void *arg)
 }
 
 void
-OTF_perror (char *prefix)
+OTF_perror (const char *prefix)
 {
   if (OTF_error == 0)
     sprintf (error_message, "%s", error_string[0]);
