@@ -24,5 +24,5 @@ write to the Free Software Foundation, Inc., 59 Temple Place, Suite
 #define OTF_ERROR(err, arg)	\
   return (otf__error ((err), errfmt, (arg)), errret)
 
-extern int otf__error (int err, char *fmt, void *arg);
+extern int otf__error (int err, const char *fmt, const void *arg);
 
